import mock
from requests.exceptions import ConnectionError, ReadTimeout
from wukong.request import SolrRequest
from wukong.errors import *
import pytest
import json

try:
    import unittest2 as unittest
except ImportError:
    import unittest

class Response(object):
    pass

class TestSolrRequest(unittest.TestCase):

    client = SolrRequest(["http://localsolr:7070/solr/","http://localsolr:8080/solr/"])

    def test_request_post(self):
        with mock.patch('wukong.request.SolrRequest.request') as mock_request:
         	self.client.post(
         		"fake_path",
         		params={
         			"fake_param": "fake_value"
         		},
         		body={
         			"fake_data": "fake_value"
         		}
         	)

        mock_request.assert_called_once_with(
            'fake_path', {"fake_param": "fake_value"}, 'POST',
            body={
     			"fake_data": "fake_value"
     		}
        )

    def test_request_get(self):
        with mock.patch('wukong.request.SolrRequest.request') as mock_request:
         	self.client.get(
         		"fake_path",
         		params={
         			"fake_param": "fake_value"
         		}
         	)

        mock_request.assert_called_once_with(
                'fake_path', {"fake_param": "fake_value"}, 'GET'
        )

    def test_request_request(self):
    	client = SolrRequest(["http://localsolr:8080/solr/"])

    	with mock.patch('requests.sessions.Session.request') as mock_request:
            fake_response =  Response()
            fake_response.status_code = 200
            fake_response.text = json.dumps({'fake_data': 'fake_value'})
            mock_request.return_value = fake_response
            response = client.request(
            	'fake_path',
            	{"fake_params": "fake_value"},
            	'GET',
            	body={"fake_body": "fake_value"}
            )

            mock_request.assert_called_once_with(
                'GET', 'http://localsolr:8080/solr/fake_path',
                params={
                	"fake_params": "fake_value",
                	'wt': 'json',
                	'omitHeader': 'true',
                	'json.nl': 'map'
                },
                headers={'content-type': 'application/json'},
                data={"fake_body": "fake_value"},
                timeout=15
            )

            assert response == {'fake_data': 'fake_value'}

    def test_request_request__empty_params(self):
        client = SolrRequest(["http://localsolr:8080/solr/"])

        with mock.patch('requests.sessions.Session.request') as mock_request:
            fake_response =  Response()
            fake_response.status_code = 200
            fake_response.text = json.dumps({'fake_data': 'fake_value'})
            mock_request.return_value = fake_response
            response = client.request(
                'fake_path',
                None,
                'GET',
                body={"fake_body": "fake_value"}
            )

        mock_request.assert_called_once_with(
            'GET', 'http://localsolr:8080/solr/fake_path',
            params={
                'wt': 'json',
                'omitHeader': 'true',
                'json.nl': 'map'
            },
            headers={'content-type': 'application/json'},
            data={"fake_body": "fake_value"},
            timeout=15
        )


    def test_request_request__status_code(self):

        with mock.patch('requests.sessions.Session.request') as mock_request:
            fake_response =  Response()
            fake_response.status_code = 400
            fake_response.reason = "Test Error"
            mock_request.return_value = fake_response
            with self.assertRaises(SolrError) as cm:
                response = self.client.request(
                    'fake_path',
                    {"fake_params": "fake_value"},
                    'GET',
                    body={"fake_body": "fake_value"}
                )

            mock_request.assert_any_call(
                'GET', 'http://localsolr:8080/solr/fake_path',
                params={
                    "fake_params": "fake_value",
                    'wt': 'json',
                    'omitHeader': 'true',
                    'json.nl': 'map'
                },
                headers={'content-type': 'application/json'},
                data={"fake_body": "fake_value"},
                timeout=15
            )

            mock_request.assert_any_call(
                'GET', 'http://localsolr:7070/solr/fake_path',
                params={
                    "fake_params": "fake_value",
                    'wt': 'json',
                    'omitHeader': 'true',
                    'json.nl': 'map'
                },
                headers={'content-type': 'application/json'},
                data={"fake_body": "fake_value"},
                timeout=15
            )

            solr_error = cm.exception
            self.assertEqual(str(solr_error), "Test Error" )

    def test_request_request__malformed_response(self):
        client = SolrRequest(["http://localsolr:8080/solr/"])

        with mock.patch('requests.sessions.Session.request') as mock_request:
            fake_response =  Response()
            fake_response.status_code = 200
            fake_response.text = "Malformed Response"
            mock_request.return_value = fake_response
            with self.assertRaises(SolrError) as cm:
                response = client.request(
                    'fake_path',
                    {"fake_params": "fake_value"},
                    'GET',
                    body={"fake_body": "fake_value"}
                )

            mock_request.assert_called_once_with(
                'GET', 'http://localsolr:8080/solr/fake_path',
                params={
                    "fake_params": "fake_value",
                    'wt': 'json',
                    'omitHeader': 'true',
                    'json.nl': 'map'
                },
                headers={'content-type': 'application/json'},
                data={"fake_body": "fake_value"},
                timeout=15
            )

            solr_error = cm.exception
            self.assertEqual(str(solr_error), "Parsing Error: Malformed Response" )

    def test_request_request__server_down(self):

        with mock.patch('requests.sessions.Session.request') as mock_request:
            def request(*args, **kwargs):
                raise ConnectionError("Server down!")

            mock_request.side_effect = request
            with self.assertRaises(SolrError) as cm:
                response = self.client.request(
                    'fake_path',
                    {"fake_params": "fake_value"},
                    'GET',
                    body={"fake_body": "fake_value"}
                )

            mock_request.assert_any_call(
                'GET', 'http://localsolr:8080/solr/fake_path',
                params={
                    "fake_params": "fake_value",
                    'wt': 'json',
                    'omitHeader': 'true',
                    'json.nl': 'map'
                },
                headers={'content-type': 'application/json'},
                data={"fake_body": "fake_value"},
                timeout=15
            )
            mock_request.assert_any_call(
                'GET', 'http://localsolr:7070/solr/fake_path',
                params={
                    "fake_params": "fake_value",
                    'wt': 'json',
                    'omitHeader': 'true',
                    'json.nl': 'map'
                },
                headers={'content-type': 'application/json'},
                data={"fake_body": "fake_value"},
                timeout=15
            )


            solr_error = cm.exception
            self.assertEqual(str(solr_error), "Server down!" )



from mock import MagicMock, patch
from wukong.zookeeper import Zookeeper
import requests

try:
    import unittest2 as unittest
except ImportError:
    import unittest


class TestZookeeper(unittest.TestCase):

    zook_client = Zookeeper("http://localzook01:2181,http://localzook02:2181")

    @patch('kazoo.client.KazooClient')
    def test_valid_clusterstate_file(self, mock_kazoo):

        class MockKazoo(object):
            def __init__(self):
                pass

            def start(self, *args, **kwargs):
                return True

            def get_children(self, *args, **kwargs):
                return ['acsvc_test_collection']

            def get(self, *args, **kwargs):
                return ('{\n  "acsvc_test_collection":{\n    "shards":{"shard1":{\n        "range":"80000000-7fffffff",\n        "state":"active",\n        "replicas":{\n          "core_node1":{\n            "state":"active",\n            "core":"acsvc_test_collection_shard1_replica2",\n            "node_name":"mt3-bmsolr102:8080_solr",\n            "base_url":"http://mt3-bmsolr102:8080/solr"},\n          "core_node2":{\n            "state":"active",\n            "core":"acsvc_test_collection_shard1_replica1",\n            "node_name":"mt3-bmsolr101:8080_solr",\n            "base_url":"http://mt3-bmsolr101:8080/solr",\n            "leader":"true"}}}},\n    "maxShardsPerNode":"1",\n    "router":{"name":"compositeId"},\n    "replicationFactor":"2",\n    "autoAddReplicas":"false"},\n  "psvc_test_collection":{\n    "shards":{"shard1":{\n        "range":"80000000-7fffffff",\n        "state":"active",\n        "replicas":{\n          "core_node1":{\n            "state":"active",\n            "core":"psvc_test_collection_shard1_replica2",\n            "node_name":"mt3-bmsolr102:8080_solr",\n            "base_url":"http://mt3-bmsolr102:8080/solr"},\n          "core_node2":{\n            "state":"active",\n            "core":"psvc_test_collection_shard1_replica1",\n            "node_name":"mt3-bmsolr101:8080_solr",\n            "base_url":"http://mt3-bmsolr101:8080/solr",\n            "leader":"true"}}}},\n    "maxShardsPerNode":"1",\n    "router":{"name":"compositeId"},\n    "replicationFactor":"2",\n    "autoAddReplicas":"false"},\n  "bm_solr_entity_collection":{\n    "shards":{"shard1":{\n        "range":"80000000-7fffffff",\n        "state":"active",\n        "replicas":{\n          "core_node1":{\n            "state":"active",\n            "core":"bm_solr_entity_collection_shard1_replica1",\n            "node_name":"mt3-bmsolr101:8080_solr",\n            "base_url":"http://mt3-bmsolr101:8080/solr",\n            "leader":"true"},\n          "core_node2":{\n            "state":"active",\n            "core":"bm_solr_entity_collection_shard1_replica2",\n            "node_name":"mt3-bmsolr102:8080_solr",\n            "base_url":"http://mt3-bmsolr102:8080/solr"}}}},\n    "maxShardsPerNode":"1",\n    "router":{"name":"compositeId"},\n    "replicationFactor":"2",\n    "autoAddReplicas":"false"},\n  "qb_category_collection":{\n    "shards":{"shard1":{\n        "range":"80000000-7fffffff",\n        "state":"active",\n        "replicas":{\n          "core_node1":{\n            "state":"active",\n            "core":"qb_category_collection_shard1_replica1",\n            "node_name":"mt3-bmsolr101:8080_solr",\n            "base_url":"http://mt3-bmsolr101:8080/solr",\n            "leader":"true"},\n          "core_node2":{\n            "state":"active",\n            "core":"qb_category_collection_shard1_replica2",\n            "node_name":"mt3-bmsolr102:8080_solr",\n            "base_url":"http://mt3-bmsolr102:8080/solr"}}}},\n    "maxShardsPerNode":"1",\n    "router":{"name":"compositeId"},\n    "replicationFactor":"2",\n    "autoAddReplicas":"false"},\n  "mt3_qb_category_collection":{\n    "shards":{"shard1":{\n        "range":"80000000-7fffffff",\n        "state":"active",\n        "replicas":{\n          "core_node1":{\n            "state":"active",\n            "core":"mt3_qb_category_collection_shard1_replica1",\n            "node_name":"mt3-bmsolr102:8080_solr",\n            "base_url":"http://mt3-bmsolr102:8080/solr"},\n          "core_node2":{\n            "state":"active",\n            "core":"mt3_qb_category_collection_shard1_replica2",\n            "node_name":"mt3-bmsolr101:8080_solr",\n            "base_url":"http://mt3-bmsolr101:8080/solr",\n            "leader":"true"}}}},\n    "maxShardsPerNode":"1",\n    "router":{"name":"compositeId"},\n    "replicationFactor":"2",\n    "autoAddReplicas":"false"},\n  "mt3_qb_question_collection":{\n    "shards":{"shard1":{\n        "range":"80000000-7fffffff",\n        "state":"active",\n        "replicas":{\n          "core_node1":{\n            "state":"active",\n            "core":"mt3_qb_question_collection_shard1_replica1",\n            "node_name":"mt3-bmsolr102:8080_solr",\n            "base_url":"http://mt3-bmsolr102:8080/solr"},\n          "core_node2":{\n            "state":"active",\n            "core":"mt3_qb_question_collection_shard1_replica2",\n            "node_name":"mt3-bmsolr101:8080_solr",\n            "base_url":"http://mt3-bmsolr101:8080/solr",\n            "leader":"true"}}}},\n    "maxShardsPerNode":"1",\n    "router":{"name":"compositeId"},\n    "replicationFactor":"2",\n    "autoAddReplicas":"false"},\n  "mt2_qb_category_collection":{\n    "shards":{"shard1":{\n        "range":"80000000-7fffffff",\n        "state":"active",\n        "replicas":{\n          "core_node1":{\n            "state":"active",\n            "core":"mt2_qb_category_collection_shard1_replica1",\n            "node_name":"mt3-bmsolr102:8080_solr",\n            "base_url":"http://mt3-bmsolr102:8080/solr"},\n          "core_node2":{\n            "state":"active",\n            "core":"mt2_qb_category_collection_shard1_replica2",\n            "node_name":"mt3-bmsolr101:8080_solr",\n            "base_url":"http://mt3-bmsolr101:8080/solr",\n            "leader":"true"}}}},\n    "maxShardsPerNode":"1",\n    "router":{"name":"compositeId"},\n    "replicationFactor":"2",\n    "autoAddReplicas":"false"},\n  "mt2_qb_question_collection":{\n    "shards":{"shard1":{\n        "range":"80000000-7fffffff",\n        "state":"active",\n        "replicas":{\n          "core_node1":{\n            "state":"active",\n            "core":"mt2_qb_question_collection_shard1_replica2",\n            "node_name":"mt3-bmsolr102:8080_solr",\n            "base_url":"http://mt3-bmsolr102:8080/solr"},\n          "core_node2":{\n            "state":"active",\n            "core":"mt2_qb_question_collection_shard1_replica1",\n            "node_name":"mt3-bmsolr101:8080_solr",\n            "base_url":"http://mt3-bmsolr101:8080/solr",\n            "leader":"true"}}}},\n    "maxShardsPerNode":"1",\n    "router":{"name":"compositeId"},\n    "replicationFactor":"2",\n    "autoAddReplicas":"false"},\n  "qb_question_test_collection":{\n    "shards":{"shard1":{\n        "range":"80000000-7fffffff",\n        "state":"active",\n        "replicas":{\n          "core_node1":{\n            "state":"active",\n            "core":"qb_question_test_collection_shard1_replica2",\n            "node_name":"mt3-bmsolr101:8080_solr",\n            "base_url":"http://mt3-bmsolr101:8080/solr",\n            "leader":"true"},\n          "core_node2":{\n            "state":"active",\n            "core":"qb_question_test_collection_shard1_replica1",\n            "node_name":"mt3-bmsolr102:8080_solr",\n            "base_url":"http://mt3-bmsolr102:8080/solr"}}}},\n    "maxShardsPerNode":"1",\n    "router":{"name":"compositeId"},\n    "replicationFactor":"2",\n    "autoAddReplicas":"false"}}', )

            def stop(self):
                return True

        mock_kazoo.return_value = MockKazoo()
        result = self.zook_client.get_active_hosts()
        self.assertEqual(result.sort(),
                         [u'http://mt3-bmsolr102:8080',
                          u'http://mt3-bmsolr101:8080'].sort())

    @patch('kazoo.client.KazooClient')
    def test_no_clusterstate_file(self, mock_kazoo):

        class MockKazoo(object):
            def __init__(self):
                pass

            def start(self, *args, **kwargs):
                return True

            def get_children(self, *args, **kwargs):
                return []

            def get(self, *args, **kwargs):
                return None

            def stop(self):
                return True

        mock_kazoo.return_value = MockKazoo()
        result = self.zook_client.get_active_hosts()
        self.assertEqual(result, [])

    @patch('kazoo.client.KazooClient')
    def test_bad_connection(self, mock_kazoo):

        class MockKazoo(object):
            def __init__(self):
                pass

            def start(self, *args, **kwargs):
                raise requests.exceptions.ConnectionError

            def get_children(self, *args, **kwargs):
                return []

            def get(self, *args, **kwargs):
                return None

            def stop(self):
                return True

        mock_kazoo.return_value = MockKazoo()
        result = self.zook_client.get_active_hosts()
        self.assertEqual(result, [])



import mock
from mock import PropertyMock
from wukong.query import *
from wukong.errors import *
import pytest

from parameterizedtestcase import ParameterizedTestCase

try:
    import unittest2 as unittest
except ImportError:
    import unittest

class TestNode(ParameterizedTestCase):
    def test_node_comp_node_init(self):
        node = Comparator("eq", "name", "Test Name")

        assert node.operator == "eq"

        assert node.key == "name"

        assert node.value == "Test Name"

    @ParameterizedTestCase.parameterize(
        ('operator', 'key', 'value', 'expected_query', 'expected_error'),
        (
            ("eq", "name", "Test Name", "name:\"Test Name\"", None),
            ("eq", "name", 100, "name:100", None),
            ("eq", "name", None, "-name:[* TO *]", None),
            ("in", "name", ["Test1", "Test2"], "name:(\"Test1\" \"Test2\")", None),
            ("in", "name", [1, 2], "name:(1 2)", None),
            ("in", "name", ["Test1", None], "name:(\"Test1\")", None),
            ("in", "name", "Test1", None, "Operator:in only takes list type"),
            ("in", "name", 1, None, "Operator:in only takes list type"),
            ("ne", "name", "Test Name", "-name:\"Test Name\"", None),
            ("ne", "name", 100, "-name:100", None),
            ("ne", "name", None, "name:*", None),
            ("wc", "name", "Test Name", "name:*Test Name*", None),
            ("wc", "name", "Test*", "name:Test*", None),
            ("wc", "name", "*Test", "name:*Test", None),
            ("wc", "name", None, "name:*", None),
            ("ge", "name", 100, "name:[100 TO *]", None),
            ("ge", "name", None, "name:[* TO *]", None),
            ("le", "name", 100, "name:[* TO 100]", None),
            ("le", "name", None, "name:[* TO *]", None),
            ("op", "name", "Test", None, "The operator:op is not supported"),
            ("op", None, "Test", "", None)
        )
    )
    def test_node_comp_node_parsed_solr_query(self, operator, key, value,
        expected_query, expected_error):
        node = Comparator(operator, key, value)

        try:
            parsed_query = node.parsed_solr_query
        except SolrError as e:
            if expected_error:
                assert str(e) == expected_error
        else:
            assert parsed_query == expected_query

    @ParameterizedTestCase.parameterize(
        ('args', 'kwargs', 'expected_query', 'expected_error'),
        (
            (AND, [], {"name__eq":"Test Name","city__eq":"Test City"},
            ["(name:\"Test Name\" AND city:\"Test City\")",
            "(city:\"Test City\" AND name:\"Test Name\")"], None),
            (AND, [], {"name__nin":["name1", "name2"],"city__eq":"Test City"},
             ['(city:"Test City" AND -name:("name1" "name2"))',
              '(-name:("name1" "name2") AND city:"Test City")'],
             None),
            (AND, [], {"name__nin":["name1", 2],"city__eq":"Test City"},
             ['(city:"Test City" AND -name:("name1" 2))',
              '(-name:("name1" 2) AND city:"Test City")'],
             None),
            (AND, [], {"name__nin":"name1", "city__eq":"Test City"},
             ['(city:"Test City" AND -name:("name1"))'],
             'Operator:not in only takes list type'),
            (AND, [], {"name__in":["name1", "name2"],"city__eq":"Test City"},
             ['(city:"Test City" AND name:("name1" "name2"))',
              '(name:("name1" "name2") AND city:"Test City")'],
             None),
            (AND, [], {"name__ex":"test","city__eq":"Test City"},
             ['(city:"Test City" AND name:[* TO *])',
              '(name:[* TO *] AND city:"Test City")'],
             None),
            (AND, [], {"name__nex":"test","city__eq":"Test City"},
             ['(-name:[* TO *] AND city:"Test City")',
              '(city:"Test City" AND -name:[* TO *])'],
             None),
            (AND, [], {"name_eq":"Test Name"},
            None, "The operator is not specified in name_eq"),
            (AND, [Comparator("eq","name", "Test Name")],{},
            ["name:\"Test Name\""], None),
            (AND, [Comparator("le","size", 999)],{},
            ["size:[* TO 999]"], None),
            (AND, [AND(name__eq="Test Name")],
            {"city__in":["Test City1", "Test City2"]},
            ["(name:\"Test Name\" AND city:(\"Test City1\" \"Test City2\"))",
            "(city:(Test City1,Test City2) AND name:\"Test Name\")"], None),
            (OR, [], {"name__eq":"Test Name","city__ge":1},
            ["(name:\"Test Name\" OR city:[1 TO *])",
            "(city:[1 TO *] OR name:\"Test Name\")"], None),
            (OR, [], {"testkey":"Test Name"},
            None, "The operator is not specified in testkey"),
            (OR, [Comparator("eq","size", 500)],{},
            ["size:500"], None),
            (OR, [Comparator("wc","name", "Test Name")],{},
            ["name:*Test Name*"], None),
            (OR, [OR(name__eq="Test Name")], {"city__wc":"*City"},
            ["(name:\"Test Name\" OR city:*City)",
            "(city:*City OR name:\"Test Name\")"], None),
            (AND, [NOT(OR(name__eq="Test Name", size__ge=100))],
            {"city__eq":"Test City"},
            ["(!((name:\"Test Name\" OR size:[100 TO *])) AND city:\"Test City\")",
            "(!((size:[100 TO *] OR name:\"Test Name\")) AND city:\"Test City\")",
            "(city:\"Test City\" AND !((name:\"Test Name\" OR size:[100 TO *])))",
            "(city:\"Test City\" AND !((size:[100 TO *] OR name:\"Test Name\")))"],
            None),
            (OR, [NOT(AND(name__eq="Test Name", size__ge=100))],
            {"city__eq":"Test City"},
            ["(!((name:\"Test Name\" AND size:[100 TO *])) OR city:\"Test City\")",
            "(!((size:[100 TO *] AND name:\"Test Name\")) OR city:\"Test City\")",
            "(city:\"Test City\" OR !((name:\"Test Name\" AND size:[100 TO *])))",
            "(city:\"Test City\" OR !((size:[100 TO *] AND name:\"Test Name\")))"],
            None)
        )
    )
    def test_node_and_or_node_parsed_solr_query(self, node_class, args, kwargs,
        expected_queries, expected_error):
        try:
            node = node_class(*args, **kwargs)
            parsed_query = node.parsed_solr_query
        except SolrError as e:
            if expected_error:
                assert str(e) == expected_error
        else:
            assert parsed_query in expected_queries

    @ParameterizedTestCase.parameterize(
        ('args', 'kwargs', 'expected_query', 'expected_error'),
        (
            ([], {}, [""], None),
            ([Comparator("eq","name", "Test Name")],{},
            ["!(name:\"Test Name\")"], None),
             ([], {"name__eq":"Test Name","city__eq":"Test City"},
            None, "Logic node:NOT only supports one operand"),
            ([Comparator("eq","name", "Test Name")], {"name__eq":"Test Name"},
            None, "Logic node:NOT only supports one operand"),
            ([Comparator("eq","name", "Test Name"),Comparator("eq","name", "Test Name")], {},
            None, "Logic node:NOT only supports one operand"),
            ([AND(name__eq="Test Name",city__in=["Test City1", "Test City2"])],{},
            ["!((name:\"Test Name\" AND city:(\"Test City1\" \"Test City2\")))",
            "!((city:(\"Test City1\" \"Test City2\") AND name:\"Test Name\"))"], None),
            ([OR(name__eq="Test Name",city__in=["Test City1", "Test City2"])],{},
            ["!((name:\"Test Name\" OR city:(\"Test City1\" \"Test City2\")))",
            "!((city:(\"Test City1\" \"Test City2\") OR name:\"Test Name\"))"], None),
        )
    )
    def test_node_not_node_parsed_solr_query(self, args, kwargs,
        expected_queries, expected_error):
        try:
            node = NOT(*args, **kwargs)
            parsed_query = node.parsed_solr_query
        except SolrError as e:
            if expected_error:
                assert str(e) == expected_error
        else:
            assert parsed_query in expected_queries

    @ParameterizedTestCase.parameterize(
        ('key', 'operator', 'value','expected_query', 'expected_error'),
        (
            (None, None, "Test Text", "", None),
            ("name", "eq", "Test Text", "name:\"Test Text\"", None),
            ("name", "eq", None, "-name:[* TO *]", None),
            ("name", "eq", 100, "name:100", None),
            ("name", "in", ["a","b"], "name:(\"a\" \"b\")", None),
            ("name", "in", [1,2], "name:(1 2)", None),
            ("name", "in", ["a",None], "name:(\"a\")", None),
            ("name", "in", "not list", None, "Operator:in only takes list type"),
            ("name", "ne", "Test Text", "-name:\"Test Text\"", None),
            ("name", "ne", None, "name:*", None),
            ("name", "ne", 100, "-name:100", None),
            ("name", "wc", "test", "name:*test*", None),
            ("name", "wc", "test*", "name:test*", None),
            ("name", "wc", "*test", "name:*test", None),
            ("name", "wc", None, "name:*", None),
            ("name", "nwc", "test*", "-name:test*", None),
            ("name", "nwc", "test", "-name:*test*", None),
            ("name", "wc", "", "name:*", None),
            ("name", "ge", 100, "name:[100 TO *]", None),
            ("name", "ge", "a", "name:[a TO *]", None),
            ("name", "ge", None, "name:[* TO *]", None),
            ("name", "ge", "", "name:[* TO *]", None),
            ("name", "le", 100, "name:[* TO 100]", None),
            ("name", "le", "a", "name:[* TO a]", None),
            ("name", "le", None, "name:[* TO *]", None),
            ("name", "le", "", "name:[* TO *]", None),
            ("size", "l", 10, "size:[* TO 10}", None),
            ("size", "le", 10, "size:[* TO 10]", None),
            ("size", "g", 10, "size:{10 TO *]", None),
            ("size", "ge", 10, "size:[10 TO *]", None),
        )
    )
    def test_node_comparator_parsed_solr_query(self, key, operator,
        value, expected_query, expected_error):
        try:
            node = Comparator(operator, key, value)
            parsed_query = node.parsed_solr_query
        except SolrError as e:
            if expected_error:
                assert str(e) == expected_error
        else:
            assert parsed_query == expected_query



import mock
from wukong.api import SolrAPI
from wukong.errors import *
import json

try:
    import unittest2 as unittest
except ImportError:
    import unittest

class TestSolrAPI(unittest.TestCase):

    api = SolrAPI(
        "localsolr:7070,localsolr:8080",
        "test_collection",
        "localzook01:2181,localzook02:2181"
    )

    def test_api_constructor___one_node(self):
        api = SolrAPI(
            "localsolr:8080",
            "test_collection",
            "localzook01:2181,localzook02:2181"
        )

        assert api.solr_hosts == \
               ["http://localsolr:8080/solr/"]

        assert api.solr_collection == "test_collection"

    def test_api_constructor__node_list(self):
        api = SolrAPI(
            "localsolr:7070,localsolr:8080",
            "test_collection",
            "localzook01:2181,localzook02:2181"
        )

        assert api.solr_hosts == \
               ["http://localsolr:7070/solr/","http://localsolr:8080/solr/"]

        assert api.solr_collection == "test_collection"

    def test_api_constructor__error_no_solr_host(self):
        with self.assertRaises(SolrError) as cm:
            SolrAPI(
                None,
                "test_collection"
            )

        solr_error = cm.exception
        self.assertEqual(str(solr_error), "Either solr_hosts or solr_collection can not be None")

    def test_api_constructor__no_zook_host(self):
        api = SolrAPI(
            "localsolr:7070,localsolr:8080",
            "test_collection",
            None
        )

        assert api.solr_hosts == \
               ["http://localsolr:7070/solr/","http://localsolr:8080/solr/"]
        assert api.solr_collection == "test_collection"
        assert api.zookeeper_hosts == None

    def test_api_constructor__only_zook_host(self):
        api = SolrAPI(
            None,
            'test_collection',
            'localzook:2181,localzook:2182/solr'
        )

        self.assertIsNotNone(api.solr_hosts)

    def test_api_constructor__error_no_collection(self):
        with self.assertRaises(SolrError) as cm:
            SolrAPI(
                "localsolr:8080",
                None,
                "localzook01:2181,localzook02:2181"
            )

        solr_error = cm.exception
        self.assertEqual(str(solr_error), "Either solr_hosts or solr_collection can not be None")

    def test_api_update(self):
        with mock.patch('wukong.request.SolrRequest.post') as mock_method:
            mock_method.return_value={}

            docs = [
                {
                    "pk": "Test PK 1",
                    "test_field": "Test Value 1"
                },
                {
                    "pk": "Test PK 2",
                    "test_field": "Test Value 2"
                }
            ]

            self.api.update(docs, commit=True)

            mock_method.assert_called_once_with(
                'test_collection/update/json',
                body=json.dumps(docs),
                params={"commit": "true"}
            )

    def test_api_update__no_docs(self):
        with mock.patch('wukong.request.SolrRequest.post') as mock_method:

            mock_method.return_value={}

            docs = []

            self.api.update(docs)

            assert not mock_method.called

    def test_api_select(self):
        with mock.patch('wukong.request.SolrRequest.get') as mock_method:
            fake_response = {
                "response":{
                    "docs":[
                        {
                            "pk": "Test PK",
                            "test_field": "Test Value"
                        }
                    ]
                }
            }

            mock_method.return_value = fake_response

            query_dict = {
                "q": "test_field:test_value",
                "rows": 10
            }
            result = self.api.select(query_dict, extra="extra_value")
            mock_method.assert_called_once_with(
                'test_collection/select',
                params={
                    "q":"test_field:test_value",
                    "rows":10,
                    "extra": "extra_value"
                }
            )
            self.assertEqual(result['docs'][0]["pk"], "Test PK")
            self.assertEqual(result['docs'][0]["test_field"], "Test Value")

    def test_api_select__group_by(self):
        with mock.patch('wukong.request.SolrRequest.get') as mock_method:
            fake_response = {
                "grouped":{
                    "city":{
                        "groups":[
                            {
                                "groupValue": 100,
                                "doclist": []
                            }
                        ]
                    }
                }
            }

            mock_method.return_value = fake_response

            query_dict = {
                "q": "test_field:test_value",
                "rows": 10,
                "group": "on",
                "group.field": "city"
            }
            result = self.api.select(query_dict, groups=True, extra="extra_value")
            mock_method.assert_called_once_with(
                'test_collection/select',
                params={
                    "q":"test_field:test_value",
                    "rows":10,
                    "group": "on",
                    "group.field": "city",
                    "extra": "extra_value"
                }
            )

            self.assertEqual(result['groups']['city']['groups'][0]["groupValue"], 100)
            self.assertEqual(result['groups']['city']['groups'][0]["doclist"], [])

    def test_api_select__group_by(self):
        with mock.patch('wukong.request.SolrRequest.get') as mock_method:
            fake_response = {
                "grouped":{
                    "city":{
                        "ngroups": 5,
                        "groups":[
                            {
                                "groupValue": 100,
                                "doclist": []
                            }
                        ]
                    }
                },
                "facet_counts":{
                    "facet_fields":{
                        "template_ids":{
                            "60": 50
                        }
                    }
                }
            }

            mock_method.return_value = fake_response

            query_dict = {
                "q": "test_field:test_value",
                "rows": 10,
                "group": "on",
                "group.field": "city",
                "facet": "on",
                "facet.field": ["template_ids"]
            }
            result = self.api.select(query_dict, groups=True, facets=True)
            mock_method.assert_called_once_with(
                'test_collection/select',
                params={
                    "q":"test_field:test_value",
                    "rows":10,
                    "group": "on",
                    "group.field": "city",
                    "facet": "on",
                    "facet.field": ["template_ids"]
                }
            )
            self.assertEqual(result['facets']['facet_fields']["template_ids"]["60"], 50)
            self.assertEqual(result['groups']['city']['groups'][0]["groupValue"], 100)
            self.assertEqual(result['groups']['city']['groups'][0]["doclist"], [])

    def test_api_delete(self):
        with mock.patch('wukong.request.SolrRequest.post') as mock_method:

            schema = self.api.delete("pk", 1, commit=True)

            mock_method.assert_called_once_with(
                'test_collection/update/json',
                body=json.dumps({
                    "delete":{
                        "query": "pk:1"
                    }
                }),
                params={"commit": "true"}
            )


    def test_api_commit(self):
        with mock.patch('wukong.request.SolrRequest.post') as mock_method:

            schema = self.api.commit()

            mock_method.assert_called_once_with(
                'test_collection/update/json',
                params={
                    "commit": "true"
                }
            )

    def test_api_get_schema_fields(self):
        with mock.patch('wukong.request.SolrRequest.get') as mock_method:

            mock_method.return_value={
                "schema": {
                    "uniqueKey": "Test PK",
                    "fields":[
                        {"name": "Test Field 1"},
                        {"name": "Test Field 2"}
                    ]
                }
            }

            schema = self.api.get_schema()

            mock_method.assert_called_once_with(
                'test_collection/schema'
            )
            self.assertEqual(schema["uniqueKey"], "Test PK")
            self.assertEqual(schema["fields"][0]["name"], "Test Field 1")
            self.assertEqual(schema["fields"][1]["name"], "Test Field 2")

    def test_api_add_schema_fields(self):
        with mock.patch('wukong.request.SolrRequest.post') as mock_method:

            mock_method.return_value={}

            fields = [{
                "name":"test_field",
                "type":"long",
                "indexed": "true",
                "stored": "true",
                "multiValued": "true"
            }]

            self.api.add_schema_fields(fields)

            mock_method.assert_called_once_with(
                'test_collection/schema/fields',
                body=json.dumps(fields)
            )

    def test_api_add_schema_fields__no_docs(self):
        with mock.patch('wukong.request.SolrRequest.post') as mock_method:

            mock_method.return_value={}

            fields = []

            self.api.add_schema_fields(fields)

            assert not mock_method.called

    def test_api_add_schema_fields__error(self):
        with mock.patch('wukong.request.SolrRequest.post') as mock_method:
            def request(*args, **kwargs):
                raise SolrError(None)

            mock_method.side_effect=request

            fields = [{
                "name":"test_field1",
                "type":"long",
                "indexed": "true",
                "stored": "true",
                "multiValued": "true"
            },{
                "name":"test_field2",
                "type":"long",
                "indexed": "true",
                "stored": "true",
                "multiValued": "true"
            }]

            with self.assertRaises(SolrSchemaUpdateError) as cm:
                self.api.add_schema_fields(fields)

        solr_error = cm.exception
        self.assertEqual(solr_error.fields, [field['name'] for field in fields])

    def test_api_is_alive(self):

        with mock.patch('wukong.request.SolrRequest.get') as mock_get:
            mock_get.return_value = {
                "znode": {
                    "data": json.dumps({
                        "test_collection": {
                            "shards":{
                                "test_collection_shard1":{
                                    "replicas":{
                                        "test_collection_shard1_replica1": {
                                            "state": "active"
                                        }
                                    }
                                }
                            }
                        },
                        "test_collection1": {
                            "shards":{
                                "test_collection1_shard1":{
                                    "replicas":{
                                        "test_collection1_shard1_replica1": {
                                            "state": "active"
                                        }
                                    }
                                }
                            }
                        }
                    })

                }

            }

            result = self.api.is_alive()

        assert result == True

    def test_api_is_alive__down(self):

        with mock.patch('wukong.request.SolrRequest.get') as mock_get:
            mock_get.return_value = {
                "znode": {
                    "data": json.dumps({
                        "test_collection": {
                            "shards":{
                                "test_collection_shard1":{
                                    "replicas":{
                                        "test_collection_shard1_replica1": {
                                            "state": "down"
                                        }
                                    }
                                }
                            }
                        },
                        "test_collection1": {
                            "shards":{
                                "test_collection1_shard1":{
                                    "replicas":{
                                        "test_collection1_shard1_replica1": {
                                            "state": "active"
                                        }
                                    }
                                }
                            }
                        }
                    })

                }

            }

            result = self.api.is_alive()

        assert result == False

    def test_api_is_alive__server_down(self):

        with mock.patch('wukong.request.SolrRequest.get') as mock_get:

            def get(*args, **kwargs):
                raise SolrError("Server down!")

            mock_get.side_effect = get

            result = self.api.is_alive()

        assert result == False

    def test_api_is_alive__malformed_response(self):

        with mock.patch('wukong.request.SolrRequest.get') as mock_get:
            mock_get.return_value = {
                "znode": {
                    "data": "Malformed Json"
                }
            }

            result = self.api.is_alive()

        assert result == False



import mock
from mock import PropertyMock
from wukong.query import *
from wukong.models import *
from wukong.errors import *
import pytest

from parameterizedtestcase import ParameterizedTestCase

try:
    import unittest2 as unittest
except ImportError:
    import unittest


class FakeSolrDoc(SolrDoc):
    collection_name = "fake_collection"
    solr_hosts = "fake_host"

class TestQuery(unittest.TestCase):
    def test_query_init__normal(self):
        node = OR(name__eq="Test Name")
        qm = SolrQueryManager(
            doc_class=FakeSolrDoc,
            node=node,
            weight_dict={"name":5},
            returned_fields=["name"],
            edismax=True,
            rows=10,
            start=5,
            sort_str="-entity_id"
        )

        assert qm.doc_class == FakeSolrDoc
        assert qm.node == node
        assert qm.weight_dict == {"name":5}
        assert qm.returned_fields == ["name"]
        assert qm.edismax == True
        assert qm.rows == 10
        assert qm.sort_str == "-entity_id"

    def test_query_init__missing_args(self):
        qm = SolrQueryManager(FakeSolrDoc)

        assert qm.doc_class == FakeSolrDoc
        assert isinstance(qm.node, AND)
        assert qm.weight_dict is None
        assert qm.returned_fields is None
        assert qm.edismax == False
        assert qm.rows == INFINITE_ROWS
        assert qm.start == 0
        assert qm.sort_str == None

    def test_query_order_by(self):
        qm = SolrQueryManager(FakeSolrDoc)

        qm = qm.sort_by("-test_name")

        assert qm.sort_str == "-test_name"

    def test_query_search(self):
        qm = SolrQueryManager(FakeSolrDoc)

        qm = qm.search("Test", name=10, city=5)

        assert qm.weight_dict == {"name":10, "city":5}
        assert qm.text_keywords == 'Test'
        assert qm.edismax

    def test_query_only_normal(self):
        qm = SolrQueryManager(FakeSolrDoc)

        qm = qm.only("name", "city")

        assert qm.returned_fields == ("name", "city")

    def test_query_limit_normal(self):
        qm = SolrQueryManager(FakeSolrDoc)

        qm = qm.limit(100)

        assert qm.rows == 100

    def test_query_limit_normal(self):
        qm = SolrQueryManager(FakeSolrDoc)

        qm = qm.offset(50)

        assert qm.start == 50

    def test_query_groupby_normal(self):
        qm = SolrQueryManager(FakeSolrDoc)

        qm = qm.group_by("city", group_limit=2)

        assert qm.group_fields == "city"
        assert qm.group_limit == 2

    def test_query_filter_default_node(self):
        qm = SolrQueryManager(FakeSolrDoc)

        qm = qm.filter(name__eq="test")

        assert qm.node.logic == 'COMP'
        assert qm.node.operator == 'eq'
        assert qm.node.key == 'name'
        assert qm.node.value == 'test'

    def test_query_filter_and_node(self):
        qm = SolrQueryManager(FakeSolrDoc)

        qm = qm.filter(AND(name__eq="Test Name",size__ge=100))

        assert qm.node.logic == 'AND'
        operators = [item.operator for item in qm.node.items]
        keys = [item.key for item in qm.node.items]
        values = [item.value for item in qm.node.items]
        assert 'eq' in  operators
        assert 'ge' in  operators
        assert 'name' in  keys
        assert 'size' in  keys
        assert "Test Name" in  values
        assert 100 in  values

    def test_query_filter_or_node(self):
        qm = SolrQueryManager(FakeSolrDoc)

        qm = qm.filter(OR(name__eq="Test Name",size__ge=100))

        assert qm.node.logic == 'OR'
        operators = [item.operator for item in qm.node.items]
        keys = [item.key for item in qm.node.items]
        values = [item.value for item in qm.node.items]
        assert 'eq' in  operators
        assert 'ge' in  operators
        assert 'name' in  keys
        assert 'size' in  keys
        assert "Test Name" in  values
        assert 100 in  values

    def test_query_filter_not_node(self):
        qm = SolrQueryManager(FakeSolrDoc)

        qm = qm.filter(NOT(name__eq="Test Name"))

        assert qm.node.logic == 'NOT'
        assert qm.node.items[0].operator == 'eq'
        assert qm.node.items[0].key == 'name'
        assert qm.node.items[0].value == 'Test Name'



    def test_query_filter_mix_node(self):
        qm = SolrQueryManager(FakeSolrDoc)

        qm = qm.filter(
            AND(
                OR(city__eq="Test City",size__le=1000),
                name__eq="Test Name"
            ),
            NOT(country__eq="Test Country"),
            state__eq="Test State",
            zip__eq="Test Zip"
        )

        assert qm.node.logic == 'AND'
        assert qm.node.items[0].logic == 'AND'
        assert qm.node.items[0].items[1].operator == 'eq'
        assert qm.node.items[0].items[1].key == 'name'
        assert qm.node.items[0].items[1].value == 'Test Name'
        assert qm.node.items[0].items[0].logic == 'OR'
        assert (
            qm.node.items[0].items[0].items[0].operator == 'eq' and
            qm.node.items[0].items[0].items[0].key == 'city' and
            qm.node.items[0].items[0].items[0].value == 'Test City' and
            qm.node.items[0].items[0].items[1].operator == 'le' and
            qm.node.items[0].items[0].items[1].key == 'size' and
            qm.node.items[0].items[0].items[1].value == 1000 or
            qm.node.items[0].items[0].items[0].operator == 'le' and
            qm.node.items[0].items[0].items[0].key == 'size' and
            qm.node.items[0].items[0].items[0].value == 1000 and
            qm.node.items[0].items[0].items[1].operator == 'eq' and
            qm.node.items[0].items[0].items[1].key == 'city' and
            qm.node.items[0].items[0].items[1].value == 'Test City'
            )
        assert qm.node.items[1].logic == 'NOT'
        assert qm.node.items[1].items[0].operator == 'eq'
        assert qm.node.items[1].items[0].key == 'country'
        assert qm.node.items[1].items[0].value == "Test Country"
        assert (
            qm.node.items[2].logic == 'COMP' and
            qm.node.items[2].operator == 'eq' and
            qm.node.items[2].key == 'state' and
            qm.node.items[2].value == "Test State" and
            qm.node.items[3].logic == 'COMP' and
            qm.node.items[3].operator == 'eq' and
            qm.node.items[3].key == 'zip' and
            qm.node.items[3].value == "Test Zip" or
            qm.node.items[3].logic == 'COMP' and
            qm.node.items[3].operator == 'eq' and
            qm.node.items[3].key == 'state' and
            qm.node.items[3].value == "Test State" and
            qm.node.items[2].logic == 'COMP' and
            qm.node.items[2].operator == 'eq' and
            qm.node.items[2].key == 'zip' and
            qm.node.items[2].value == "Test Zip"
        )


    def test_query_filter_chaining_filters(self):
        qm = SolrQueryManager(FakeSolrDoc)
        qm = qm.filter(name__eq="Test Name").filter(size__ge=100)

        assert qm.node.logic == 'AND'
        operators = [item.operator for item in qm.node.items]
        keys = [item.key for item in qm.node.items]
        values = [item.value for item in qm.node.items]
        assert 'eq' in  operators
        assert 'ge' in  operators
        assert 'name' in  keys
        assert 'size' in  keys
        assert "Test Name" in  values
        assert 100 in  values

    def test_query_filter_chaining_mix(self):
        qm = SolrQueryManager(FakeSolrDoc)
        qm = qm.filter(name__eq="Test Name").only("name").limit(100).offset(50)

        assert qm.node.logic == 'COMP'

        assert qm.node.operator == 'eq'
        assert qm.node.key == 'name'
        assert qm.node.value == 'Test Name'

        assert qm.rows == 100
        assert qm.start == 50

        assert qm.returned_fields == ("name",)

    def test_query_query_no_params(self):
        qm = SolrQueryManager(FakeSolrDoc)

        params = qm.query
        assert params['q'] == '*:*'
        assert params['rows'] == INFINITE_ROWS
        assert params['start'] == 0
        assert params['wt'] == 'json'

    def test_query_query_asc(self):
        qm = SolrQueryManager(FakeSolrDoc)
        qm = qm.sort_by("name")

        params = qm.query
        assert params['sort'] == 'name asc'

    def test_query_query_desc(self):
        qm = SolrQueryManager(FakeSolrDoc)
        qm = qm.sort_by("-name")

        params = qm.query
        assert params['sort'] == 'name desc'

    def test_query_query_chained_search(self):
        qm = SolrQueryManager(FakeSolrDoc)
        qm = qm.filter(id__eq=999).only("id","name")\
            .limit(100)\
            .offset(50)\
            .search("Test  Name ", minimum_matches="3<50%", name=10,city=5)\
            .boost_by_func('{!func}geodist()', bf_weight=2)\
            .boost_by_query('id:100', bq_weight=2)\
            .stats(['country'])\
            .group_by('city', group_limit=2).sort_by("-name")

        params = qm.query
        assert params['sort'] == 'name desc'
        assert params['defType'] == 'edismax'
        assert params['rows'] == 100
        assert params['start'] == 50
        assert params['q'] == 'Test Name'
        assert params['fq'] ==  'id:999'
        assert params['wt'] == 'json'
        assert params['qf'] in ['city^5 name^10', 'name^10 city^5']
        assert params['fl'] in ['id,name', 'name,id']
        assert params['group'] == 'on'
        assert params['group.ngroups'] == "true"
        assert params['group.field'] == 'city'
        assert params['stats'] == 'true'
        assert params['stats.field'] == ['country']
        assert params['bf'] == '{!func}geodist()^2'
        assert params['bq'] == 'id:100^2'
        assert params['mm'] == '3<50%'

    def test_query_with_stats_two_fields(self):
        qm = SolrQueryManager(FakeSolrDoc)
        qm = qm.stats(['country', 'city'])
        params = qm.query

        assert params['stats'] == 'true'
        assert params['stats.field'] == ['country', 'city']

    def test_query_query_chained_regular(self):
        qm = SolrQueryManager(FakeSolrDoc)
        qm = qm.filter(id__eq=999).only("id","name")\
            .limit(100)\
            .offset(50)\
            .boost_by_func('{!func}geodist()', bf_weight=2)\
            .boost_by_query('id:100', bq_weight=2)\
            .group_by('city', group_limit=2, facet="true")\
            .facet(['country'], mincount=3)\
            .stats(['country'])\
            .sort_by("-name")

        params = qm.query
        assert params['sort'] == 'name desc'
        assert params['rows'] == 100
        assert params['start'] == 50
        assert params['q'] ==  'id:999'
        assert params['wt'] == 'json'
        assert params['fl'] in ['id,name', 'name,id']
        assert params['group'] == 'on'
        assert params['group.field'] == 'city'
        assert params['group.limit'] == 2
        assert params['group.facet'] == 'true'
        assert params['facet'] == 'on'
        assert params['facet.field'] == ['country']
        assert params['facet.mincount'] == 3
        assert params['stats'] == 'true'
        assert params['stats.field'] == ['country']
        assert params['bf'] == '{!func}geodist()^2'
        assert params['bq'] == 'id:100^2'

    def test_query_query__facet_group_off1(self):
        qm = SolrQueryManager(FakeSolrDoc)
        qm = qm.filter(id__eq=999)\
            .group_by('city', group_limit=2)\
            .facet(['country'], mincount=3, group=False)\

        params = qm.query
        assert 'group.facet' not in params

    def test_query_query__facet_group_off2(self):
        qm = SolrQueryManager(FakeSolrDoc)
        qm = qm.filter(id__eq=999)\
            .group_by('city', group_limit=2)\

        params = qm.query
        assert 'group.facet' not in params

    def test_query_query__facet_group_off3(self):
        qm = SolrQueryManager(FakeSolrDoc)
        qm = qm.filter(id__eq=999)\
            .facet(['country'], mincount=3)\

        params = qm.query
        assert 'group.facet' not in params


    def test_query_fetch_all_normal(self):
        with mock.patch('wukong.api.SolrAPI.select') as mock_select:
            mock_select.return_value = {
                'docs': [
                    {
                        "id": 1,
                        "name": "Test Name 1"
                    },
                    {
                        "id": 2,
                        "name": "Test Name 2"
                    }
                ]
            }
            with mock.patch('wukong.api.SolrAPI.get_schema') as mock_schema:
                mock_schema.return_value = {
                    "uniqueKey": "id",
                    "fields": [
                        {
                            "name": "id",
                            "type": "int"
                        },
                        {
                            "name": "name",
                            "type": "string"
                        }
                    ]
                }

                qm = SolrQueryManager(FakeSolrDoc)

                docs = qm.all(metadata=True)

                assert docs[0].id == 1
                assert docs[0].name == "Test Name 1"
                assert docs[1].id == 2
                assert docs[1].name == "Test Name 2"

    def test_query_fetch_all_with_error(self):
        with mock.patch('wukong.api.SolrAPI.select') as mock_select:
            def fake_select(*args, **kwargs):
                raise SolrError((4, "Select fetch failed"))

            mock_select.side_effect = fake_select

            with mock.patch('wukong.api.SolrAPI.get_schema') as mock_schema:
                mock_schema.return_value = {
                    "uniqueKey": "id",
                    "fields": [
                        {
                            "name": "id",
                            "type": "int"
                        },
                        {
                            "name": "name",
                            "type": "string"
                        }
                    ]
                }

                with self.assertRaises(SolrError) as cm:
                    qm = SolrQueryManager(FakeSolrDoc)
                    docs = qm.all()

                schema_error = cm.exception
                self.assertEqual(str(schema_error), "Select fetch failed")


    def test_query_fetch_one_with_one_returned(self):
        with mock.patch('wukong.api.SolrAPI.select') as mock_select:
            mock_select.return_value = {
                'docs': [
                    {
                        "id": 1,
                        "name": "Test Name 1"
                    }
                ]
            }
            with mock.patch('wukong.api.SolrAPI.get_schema') as mock_schema:
                mock_schema.return_value = {
                    "uniqueKey": "id",
                    "fields": [
                        {
                            "name": "id",
                            "type": "int"
                        }
                    ]
                }

                qm = SolrQueryManager(FakeSolrDoc)

                doc = qm.one()

                assert doc.id == 1
                assert doc.name == "Test Name 1"

    def test_query_fetch_one_with_none_returned(self):
        with mock.patch('wukong.api.SolrAPI.select') as mock_select:
            mock_select.return_value = {
                'docs': []
            }
            with mock.patch('wukong.api.SolrAPI.get_schema') as mock_schema:
                mock_schema.return_value = {
                    "uniqueKey": "id",
                    "fields": [
                        {
                            "name": "id",
                            "type": "int"
                        }
                    ]
                }

                qm = SolrQueryManager(FakeSolrDoc)

                doc = qm.one()

                assert doc is None

    def test_query_fetch_one_with_multi_returned(self):
        with mock.patch('wukong.api.SolrAPI.select') as mock_select:
            mock_select.return_value = {
                'docs': [
                    {
                        "id": 1,
                        "name": "Test Name 1"
                    },
                    {
                        "id": 2,
                        "name": "Test Name 2"
                    }
                ]
            }
            with mock.patch('wukong.api.SolrAPI.get_schema') as mock_schema:
                mock_schema.return_value = {
                    "uniqueKey": "id",
                    "fields": [
                        {
                            "name": "id",
                            "type": "int"
                        }
                    ]
                }

                qm = SolrQueryManager(FakeSolrDoc)

                doc = qm.one()

                assert doc.id == 1
                assert doc.name == "Test Name 1"

    def test_query_get(self):
        with mock.patch('wukong.api.SolrAPI.select') as mock_select:
            mock_select.return_value = {
                'docs': [
                    {
                        "id": 1,
                        "name": "Test Name 1"
                    }
                ]
            }
            with mock.patch('wukong.api.SolrAPI.get_schema') as mock_schema:
                mock_schema.return_value = {
                    "uniqueKey": "id",
                    "fields": [
                        {
                            "name": "id",
                            "type": "int"
                        }
                    ]
                }

                qm = SolrQueryManager(FakeSolrDoc)

                doc = qm.get(id__eq=1)

                assert doc.id == 1
                assert doc.name == "Test Name 1"

    def test_query_create_normal(self):
        with mock.patch('wukong.api.SolrAPI.select') as mock_select:
            mock_select.return_value = {
                'docs': []
            }
            with mock.patch('wukong.api.SolrAPI.get_schema') as mock_schema:
                mock_schema.return_value = {
                    "uniqueKey": "id",
                    "fields": [
                        {
                            "name": "id",
                            "type": "int"
                        },
                        {
                            "name": "name",
                            "type": "string"
                        }
                    ]
                }
                with mock.patch('wukong.api.SolrAPI.update') as mock_add:
                    with mock.patch('wukong.api.SolrAPI.commit') as mock_commit:
                        qm = SolrQueryManager(FakeSolrDoc)
                        doc = qm.create(id=1, name="Test Name")

                        assert doc.id == 1
                        assert doc.name == "Test Name"

    def test_query_create_duplicated_unique_ke(self):
        with mock.patch('wukong.api.SolrAPI.select') as mock_select:
            mock_select.return_value = {
                'docs': [
                    {
                        "id": 1,
                        "name": "Test Name 1"
                    }
                ]
            }
            with mock.patch('wukong.api.SolrAPI.get_schema') as mock_schema:
                mock_schema.return_value = {
                    "uniqueKey": "id",
                    "fields": [
                        {
                            "name": "id",
                            "type": "int"
                        },
                        {
                            "name": "name",
                            "type": "string"
                        }
                    ]
                }

                with self.assertRaises(SolrDuplicateUniqueKeyError) as cm:
                    qm = SolrQueryManager(FakeSolrDoc)
                    doc = qm.create(id=1, name="Test Name")

                schema_error = cm.exception
                self.assertEqual(str(schema_error), "The unique key 1 already exists" )

    def test_query_update_normal(self):
        with mock.patch('wukong.api.SolrAPI.select') as mock_select:
            mock_select.return_value =  {
                'docs': [{
                        "id": 1,
                        "name": "Test Name 1"
                    }
                ]
            }
            with mock.patch('wukong.api.SolrAPI.get_schema') as mock_schema:
                mock_schema.return_value = {
                    "uniqueKey": "id",
                    "fields": [
                        {
                            "name": "id",
                            "type": "int"
                        },
                        {
                            "name": "name",
                            "type": "string"
                        }
                    ]
                }
                with mock.patch('wukong.api.SolrAPI.update') as mock_add:
                    with mock.patch('wukong.api.SolrAPI.commit') as mock_commit:
                        qm = SolrQueryManager(FakeSolrDoc)
                        doc = qm.update(id=1, name="Test Name")

                        assert doc.id == 1
                        assert doc.name == "Test Name"

    def test_query_create_no_doc_exist(self):
        with mock.patch('wukong.api.SolrAPI.select') as mock_select:
            mock_select.return_value = {
                'docs': []
            }
            with mock.patch('wukong.api.SolrAPI.get_schema') as mock_schema:
                mock_schema.return_value = {
                    "uniqueKey": "id",
                    "fields": [
                        {
                            "name": "id",
                            "type": "int"
                        },
                        {
                            "name": "name",
                            "type": "string"
                        }
                    ]
                }

                with self.assertRaises(SolrDocumentNotExistError) as cm:
                    qm = SolrQueryManager(FakeSolrDoc)
                    doc = qm.update(id=1, name="Test Name")

                schema_error = cm.exception
                self.assertEqual(str(schema_error), "The document with unique key 1 does not exist" )



import mock
from requests.exceptions import ConnectionError
from wukong.request import SolrRequest
from wukong.errors import *
import time
import json

try:
    import unittest2 as unittest
except ImportError:
    import unittest

class Response(object):
    pass

class TestSolrZookRequest(unittest.TestCase):

    client = SolrRequest(["http://localsolr:7070/solr/","http://localsolr:8080/solr/"],
                         zookeeper_hosts=["http://localzook:2181",
                                          "http://localzook:2181"])

    def test_request_post(self):

        client = SolrRequest(["http://localsolr:7070/solr/","http://localsolr:8080/solr/"],
                             zookeeper_hosts=["http://localzook:2181", "http://localzook:2181"])

        with mock.patch('wukong.request.SolrRequest.request') as mock_request:
            client.post(
                "fake_path",
                params={
                    "fake_param": "fake_value"
                },
                body={
                    "fake_data": "fake_value"
                }
            )

        mock_request.assert_called_once_with(
            'fake_path', {"fake_param": "fake_value"}, 'POST',
            body={
                "fake_data": "fake_value"
            }
        )

    def test_request_get(self):

        client = SolrRequest(["http://localsolr:7070/solr/","http://localsolr:8080/solr/"],
                             zookeeper_hosts=["http://localzook:2181", "http://localzook:2181"])

        with mock.patch('wukong.request.SolrRequest.request') as mock_request:
            client.get(
                "fake_path",
                params={
                    "fake_param": "fake_value"
                }
            )

        mock_request.assert_called_once_with(
            'fake_path', {"fake_param": "fake_value"}, 'GET'
        )

    def test_request_request__server_down(self):

        client = SolrRequest(["http://localsolr:7070/solr/","http://localsolr:8080/solr/"],
                             zookeeper_hosts=["http://localzook:2181", "http://localzook:2181"])

        with mock.patch('requests.sessions.Session.request') as mock_request:
            def request(*args, **kwargs):
                raise ConnectionError("Server down!")

            mock_request.side_effect = request

            with mock.patch('wukong.zookeeper.Zookeeper.get_active_hosts') as mock_zookeeper:
                def get_active_hosts():
                    return ["http://localsolr:7070/solr/"]

                mock_zookeeper.side_effect = get_active_hosts

                with self.assertRaises(SolrError) as cm:
                    response = client.request(
                        'fake_path',
                        {"fake_params": "fake_value"},
                        'GET',
                        body={"fake_body": "fake_value"}
                    )

                    mock_request.assert_any_call(
                        'GET', 'http://localsolr:8080/solr/fake_path',
                        params={
                            "fake_params": "fake_value",
                            'wt': 'json',
                            'omitHeader': 'true',
                            'json.nl': 'map'
                        },
                        headers={'content-type': 'application/json'},
                        data={"fake_body": "fake_value"},
                        timeout=15
                    )

                    self.assertEqual(client.current_hosts,
                                     ["http://localsolr:7070/solr/"])

                    mock_request.assert_any_call(
                        'GET', 'http://localsolr:7070/solr/fake_path',
                        params={
                            "fake_params": "fake_value",
                            'wt': 'json',
                            'omitHeader': 'true',
                            'json.nl': 'map'
                        },
                        headers={'content-type': 'application/json'},
                        data={"fake_body": "fake_value"},
                        timeout=15
                    )

            solr_error = cm.exception
            self.assertEqual(str(solr_error), "Server down!")

    def test_request_request__all_servers_down(self):

        client = SolrRequest(["http://localsolr:7070/solr/","http://localsolr:8080/solr/"],
                             zookeeper_hosts=["http://localzook:2181", "http://localzook:2181"])

        with mock.patch('requests.sessions.Session.request') as mock_request:
            def request(*args, **kwargs):
                raise ConnectionError("Server down!")

            mock_request.side_effect = request

            with mock.patch('wukong.zookeeper.Zookeeper.get_active_hosts') as mock_zookeeper:
                def get_active_hosts():
                    return []

                mock_zookeeper.side_effect = get_active_hosts

                with self.assertRaises(SolrError) as cm:
                    response = client.request(
                        'fake_path',
                        {"fake_params": "fake_value"},
                        'GET',
                        body={"fake_body": "fake_value"}
                    )

                    self.assertEqual(client.current_hosts, [])
                    solr_error = cm.exception
                    self.assertEqual(str(solr_error),
                                     "SOLR reporting all nodes as down")

                with self.assertRaises(SolrError) as cm:
                    response2 = client.request(
                        'fake_path',
                        {"fake_params": "fake_value"},
                        'GET',
                        body={"fake_body": "fake_value"}
                    )

            solr_error = cm.exception
            self.assertEqual(str(solr_error),
                             "SOLR reporting all nodes as down")

    def test_request_refill_pool(self):

        client = SolrRequest(["http://localsolr:7070/solr/","http://localsolr:8080/solr/"],
                             zookeeper_hosts=["http://localzook:2181", "http://localzook:2181"])

        # simulate an empty pool and a 5 minute old error
        client.current_hosts = ["http://localsolr:8080/solr/"]
        client.last_error = time.time() - 5

        with mock.patch('requests.sessions.Session.request') as mock_request:
            fake_response =  Response()
            fake_response.status_code = 200
            fake_response.text = json.dumps({'fake_data': 'fake_value'})
            mock_request.return_value = fake_response
            response = client.request(
                'fake_path',
                {"fake_params": "fake_value"},
                'GET',
                body={"fake_body": "fake_value"}
            )

            self.assertEqual(client.current_hosts, client.master_hosts)

    def test_request_no_active_pool(self):

        client = SolrRequest(["http://localsolr:7070/solr/","http://localsolr:8080/solr/"],
                             zookeeper_hosts=["http://localzook:2181", "http://localzook:2181"])
        client.current_hosts = ''
        # simulate an empty pool

        with mock.patch('requests.sessions.Session.request') as mock_request:
            fake_response =  Response()
            fake_response.status_code = 200
            fake_response.text = json.dumps({'fake_data': 'fake_value'})
            mock_request.return_value = fake_response
            with self.assertRaises(SolrError) as cm:
                response = client.request(
                    'fake_path',
                    {"fake_params": "fake_value"},
                    'GET',
                    body={"fake_body": "fake_value"}
                )

            solr_error = cm.exception
            self.assertEqual(str(solr_error),
                             "SOLR reporting all nodes as down")

            # add nodes back to the bool
            client.current_hosts = ["http://localsolr:7070/solr/","http://localsolr:8080/solr/"]
            response = client.request(
                'fake_path',
                {"fake_params": "fake_value"},
                'GET',
                body={"fake_body": "fake_value"}
            )



import json

try:
    import unittest2 as unittest
except ImportError:
    import unittest


class BaseTestCase(unittest.TestCase):
    def _make_request(self):
        from pyramid.testing import DummyRequest
        request = DummyRequest()
        return request

    def _make_response(self, data, status_code=200):
        from requests.models import Response
        response = Response()
        response._content = json.dumps(data).encode('utf-8')
        response.status_code = status_code
        return response


import mock
from mock import Mock, PropertyMock, patch
from wukong.models import *
from wukong.errors import *
import pytest


try:
    import unittest2 as unittest
except ImportError:
    import unittest

class FakeSolrDoc(SolrDoc):
    solr_hosts = "fake_host"
    zookeeper_hosts = "fake_zookeeper_hosts"
    collection_name = "fake_collection"

class FakeSolrDoc1(SolrDoc):
    solr_hosts = "fake_host"
    zookeeper_hosts = "fake_zookeeper_hosts"
    collection_name = "fake_collection1"

class FakeCollection(object):
    name = "fake_collection"

mock_schema = Mock(return_value = {
    "uniqueKey": "id",
    "fields": [
        {
            "name": "id",
            "type": "int"
        },
        {
            "name": "name",
            "type": "string"
        },
        {
            "name": "zip",
            "type": "string"
        },
        {
            "name": "test_solrdoc_list",
            "type": "string",
            "multiValued": True
        }
    ],
    "dynamicFields": [
        {
            "name": "*_d1",
            "type": "int"
        },
        {
            "name": "*_d2",
            "type": "string"
        }
    ]

})

class TestModels(unittest.TestCase):
    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_init(self):
        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )

        assert doc.partial_update == True
        assert doc.field_weights == {"name":10}
        assert doc.id == 100
        assert doc.name == "Test Name"
        assert doc.collection_name == "fake_collection"
        assert doc.__class__.documents.doc_class == FakeSolrDoc
        assert doc.unique_key == "id"
        assert (doc.solr.solr_hosts ==
            ['http://fake_host/solr/'])
        assert (doc.solr.solr_collection == 'fake_collection')

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_add_schema_fields(self):
        with mock.patch('wukong.api.SolrAPI.add_schema_fields') as mock_add_fields:

            FakeSolrDoc.add_schema_fields([
                {
                    "name": "id",
                    "type": "int"
                },
                {
                    "name": "city",
                    "type": "string"
                }
            ])
            mock_add_fields.assert_called_once_with([{"name": "city","type": "string"}])

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_from_json_docs(self):

        docs = FakeSolrDoc.from_json_docs([
            {
                "id": 1,
                "name": "Test Name 1"
            },
            {
                "id": 2,
                "name": "Test Name 2"
            }
        ])
        assert docs[0].id == 1
        assert docs[0].name == "Test Name 1"
        assert docs[1].id == 2
        assert docs[1].name == "Test Name 2"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_to_str(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )

        assert str(doc)in ["{'name': 'Test Name', 'id': 100}", "{'id': 100, 'name': 'Test Name'}"]

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_repr(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )

        assert repr(doc) == "FakeSolrDoc(id=100)"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_equal_to_true(self):

        doc1 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name",
            zip="Test Zip"
        )

        doc2 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name",
            zip="Test Zip"
        )


        assert doc1 == doc2

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_equal_to_false1(self):

        doc1 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name",
            zip="Test Zip"
        )

        doc2 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )


        assert not doc1 == doc2

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_equal_to_false2(self):

        doc1 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name 1",
            zip="Test Zip"
        )

        doc2 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name 2",
            zip="Test Zip"
        )

        assert not doc1 == doc2

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_equal_to_false3(self):

        doc1 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name",
            zip="Test Zip"
        )

        doc2 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=101,
            name="Test Name",
            zip="Test Zip"
        )
        assert not doc1 == doc2

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_not_equal_to_false(self):

        doc1 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name 1",
            zip="Test Zip"
        )

        doc2 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name",
            zip="Test Zip"
        )


        assert doc1 != doc2

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_not_equal_to_true1(self):

        doc1 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name",
            zip="Test Zip"
        )

        doc2 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )


        assert doc1 != doc2

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_not_equal_to_true2(self):

        doc1 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name 1",
            zip="Test Zip"
        )

        doc2 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name 2",
            zip="Test Zip"
        )

        assert doc1 != doc2

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_not_equal_to_true3(self):

        doc1 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name",
            zip="Test Zip"
        )

        doc2 = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=101,
            name="Test Name",
            zip="Test Zip"
        )
        assert doc1 != doc2

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_set_item_update(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )

        assert doc.name == "Test Name"
        doc['name'] = "Test Name 1"
        assert doc.name == "Test Name 1"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_set_item_new(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )
        assert doc.zip == None
        doc['zip'] = "Test Zip"
        assert doc.zip ==  "Test Zip"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_get_item_none(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )
        assert doc['zip'] == None

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_get_item_exist(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )
        assert doc['name'] == "Test Name"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_get_attr_none(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )
        assert doc.zip == None

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_get_item_exist(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )
        assert doc.name == "Test Name"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_get_item_builtin_attr(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )
        assert doc.fields == {"name":"Test Name", "id":100}

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_length_method(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )
        assert len(doc.fields) == 2


    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_iter_items(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )
        fields = {}
        for name, field in doc:
            fields[name]= field

        assert fields == {"name": "Test Name", "id": 100}

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_set_field_update(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )

        assert doc.name == "Test Name"
        doc.set_field('name', "Test Name 1")
        assert doc.name == "Test Name 1"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_set_field_new(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )
        assert doc.zip == None
        doc.set_field('zip', "Test Zip")
        assert doc.zip ==  "Test Zip"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_del_field_normal(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )

        assert doc.name == "Test Name"
        doc.del_field('name')
        assert doc.name == None

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_del_field_error(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )

        with self.assertRaises(SolrDeleteUniqueKeyError) as cm:
            doc.del_field('id')

        schema_error = cm.exception
        self.assertEqual(str(schema_error), "The unique key id can not be deleted" )

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_is_partial_update(self):

        doc = FakeSolrDoc(
            partial_update=True,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )

        assert doc.is_partial_update()

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_set_partial_update(self):

        doc = FakeSolrDoc(
            partial_update=False,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )
        assert doc.partial_update == False
        doc.set_partial_update(True)
        assert doc.partial_update == True

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_set_field_weight(self):

        doc = FakeSolrDoc(
            partial_update=False,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )
        assert doc.field_weights['name'] == 10
        doc.set_field_weight('name', 20)
        assert doc.field_weights['name'] == 20


    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_set_field_weight_no_key(self):

        doc = FakeSolrDoc(
            partial_update=False,
            id=100,
            name="Test Name"
        )
        assert doc.field_weights is None
        doc.set_field_weight('name', 20)
        assert doc.field_weights['name'] == 20

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_get_field_weight(self):

        doc = FakeSolrDoc(
            partial_update=False,
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )

        assert doc.get_field_weight('name') == 10


    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_get_field_weight_no_key(self):

        doc = FakeSolrDoc(
            partial_update=False,
            id=100,
            name="Test Name"
        )

        assert doc.get_field_weight('name') is None

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_validate_schema_fields(self):

        FakeSolrDoc.validate_schema_fields({
            "id": 100,
            "name": "Test Name",
            "zip": "Test Zip",
            "test_solrdoc_list": ["test1", "test2"],
            "test_solrdoc_d1": 1,
            "test_solrdoc_d2": "test d2"
        })

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_validate_schema_fields_no_pk(self):
        with self.assertRaises(SolrSchemaValidationError) as cm:
            FakeSolrDoc.validate_schema_fields({
                "name": "Test Name",
                "zip": "Test Zip",
                "test_solrdoc_list": ["test1", "test2"],
                "test_solrdoc_d1": 1,
                "test_solrdoc_d2": "test d2"
            })

        schema_error = cm.exception
        self.assertEqual(str(schema_error), "Unique key is not specified" )

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_validate_schema_fields_none_pk(self):
        with self.assertRaises(SolrSchemaValidationError) as cm:
            FakeSolrDoc.validate_schema_fields({
                "id": None,
                "name": "Test Name",
                "zip": "Test Zip",
                "test_solrdoc_list": ["test1", "test2"],
                "test_solrdoc_d1": 1,
                "test_solrdoc_d2": "test d2"
            })

        schema_error = cm.exception
        self.assertEqual(str(schema_error), "Unique key is not specified" )

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_validate_schema_fields_multivalue_error(self):
        with self.assertRaises(SolrSchemaValidationError) as cm:
            FakeSolrDoc.validate_schema_fields({
                "id": 1,
                "name": "Test Name",
                "zip": "Test Zip",
                "test_solrdoc_list": "test1",
                "test_solrdoc_d1": 1,
                "test_solrdoc_d2": "test d2"
            })

        schema_error = cm.exception
        self.assertEqual(str(schema_error), "test_solrdoc_list is not list type" )

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_validate_schema_fields_error(self):
        with self.assertRaises(SolrSchemaValidationError) as cm:
            FakeSolrDoc.validate_schema_fields({
                "id": 1,
                "name": "Test Name",
                "zip": "Test Zip",
                "test_solrdoc_list": ["test1", "test2"],
                "test_solrdoc_d3": 1,
                "test_solrdoc_d2": "test d2"
            })

        schema_error = cm.exception
        self.assertEqual(str(str(schema_error)), "Field (test_solrdoc_d3) is not in SOLR schema fields" )

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_get_data_for_solr_full(self):

        doc = FakeSolrDoc(
            partial_update=False,
            id=100,
            name="Test Name"
        )

        data = doc.get_data_for_solr()

        assert data == {"id": 100, "name": "Test Name"}

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_get_data_for_solr_partial(self):

        doc = FakeSolrDoc(
            partial_update=True,
            id=100,
            name="Test Name"
        )

        data = doc.get_data_for_solr()

        assert data == {"id": 100, "name": {"set":"Test Name"}}

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_get_data_for_solr_weighted(self):

        doc = FakeSolrDoc(
            field_weights={"name":10},
            id=100,
            name="Test Name"
        )

        data = doc.get_data_for_solr()

        assert data == {"id": 100, "name": {"boost":10,"value":"Test Name"}}

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_index_with_no_commit(self):
        with mock.patch('wukong.api.SolrAPI.update') as mock_update:
            mock_update.return_value = {}
            with mock.patch('wukong.api.SolrAPI.commit') as mock_commit:
                mock_commit.return_value = {}

                doc = FakeSolrDoc(
                    id=100,
                    name="Test Name"
                )

                doc.index()

                mock_update.assert_called_once_with([{"id":100,"name":"Test Name"}])
                assert not mock_commit.called

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_index_with_commit(self):
        with mock.patch('wukong.api.SolrAPI.update') as mock_update:
            mock_update.return_value = {}
            with mock.patch('wukong.api.SolrAPI.commit') as mock_commit:
                mock_commit.return_value = {}

                doc = FakeSolrDoc(
                    id=100,
                    name="Test Name"
                )

                doc.index(commit=True)

                mock_update.assert_called_once_with([{"id":100,"name":"Test Name"}])
                assert mock_commit.called


    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_delete_with_no_commit(self):
        with mock.patch('wukong.api.SolrAPI.delete') as mock_delete:
            mock_delete.return_value = {}

            doc = FakeSolrDoc(
                id=100,
                name="Test Name"
            )

            doc.delete()
            mock_delete.assert_called_once_with("id", 100, commit=False)

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_delete_with_commit(self):
        with mock.patch('wukong.api.SolrAPI.delete') as mock_delete:
            mock_delete.return_value = {}

            doc = FakeSolrDoc(
                id=100,
                name="Test Name"
            )

            doc.delete(commit=True)
            mock_delete.assert_called_once_with("id", 100, commit=True)


    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_init_no_docs(self):
        docs = SolrDocs()

        assert docs.docs == []
        assert docs.doc_class == None

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_init_no_docs(self):
        solr_docs = [
            FakeSolrDoc(
                id=1,
                name="Test Name 1"
            ),
            FakeSolrDoc(
                id=2,
                name="Test Name 2"
            )
        ]
        docs = SolrDocs(docs=solr_docs)

        assert docs.docs[0].id == 1
        assert docs.docs[0].name == "Test Name 1"
        assert docs.docs[1].id == 2
        assert docs.docs[1].name == "Test Name 2"
        assert docs.doc_class == FakeSolrDoc

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_to_str(self):

        solr_docs = [
            FakeSolrDoc(
                id=1,
                name="Test Name 1"
            ),
            FakeSolrDoc(
                id=2,
                name="Test Name 2"
            )
        ]
        docs = SolrDocs(docs=solr_docs)

        assert str(docs) == "[FakeSolrDoc(id=1), FakeSolrDoc(id=2)]"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_repr(self):
        solr_docs = [
            FakeSolrDoc(
                id=1,
                name="Test Name 1"
            ),
            FakeSolrDoc(
                id=2,
                name="Test Name 2"
            )
        ]
        docs = SolrDocs(docs=solr_docs)
        assert repr(docs) == "SolrDocs(len(FakeSolrDoc)=2)"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_set_item(self):

        solr_docs = [
            FakeSolrDoc(
                id=1,
                name="Test Name 1"
            ),
            FakeSolrDoc(
                id=2,
                name="Test Name 2"
            )
        ]
        docs = SolrDocs(docs=solr_docs)

        assert docs.docs[0].name == "Test Name 1"
        docs[0] = FakeSolrDoc(
            id=3,
            name="Test Name 3"
        )
        assert docs.docs[0].name == "Test Name 3"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_get_item(self):

        solr_docs = [
            FakeSolrDoc(
                id=1,
                name="Test Name 1"
            ),
            FakeSolrDoc(
                id=2,
                name="Test Name 2"
            )
        ]
        docs = SolrDocs(docs=solr_docs)

        assert docs[0].name == "Test Name 1"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_length(self):

        solr_docs = [
            FakeSolrDoc(
                id=1,
                name="Test Name 1"
            ),
            FakeSolrDoc(
                id=2,
                name="Test Name 2"
            )
        ]
        docs = SolrDocs(docs=solr_docs)

        assert len(docs)== 2

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_iter_items(self):

        solr_docs = [
            FakeSolrDoc(
                id=1,
                name="Test Name 1"
            ),
            FakeSolrDoc(
                id=2,
                name="Test Name 2"
            )
        ]
        docs = SolrDocs(docs=solr_docs)

        solr_docs = []
        for doc in docs:
            solr_docs.append(doc)

        assert solr_docs[0].id == 1
        assert solr_docs[0].name == "Test Name 1"
        assert solr_docs[1].id == 2
        assert solr_docs[1].name == "Test Name 2"

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_add(self):
        docs = SolrDocs()
        docs.add(
            FakeSolrDoc(
                id=1,
                name="Test Name 1"
            )
        )
        docs.add(
            FakeSolrDoc(
                id=2,
                name="Test Name 2"
            )
        )
        assert len(docs.docs) == 2
        assert docs.doc_class == FakeSolrDoc

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_add_error(self):
        with self.assertRaises(SolrError) as cm:
            docs = SolrDocs()
            docs.add(
                FakeSolrDoc(
                    id=1,
                    name="Test Name 1"
                )
            )
            docs.add(
                FakeSolrDoc1(
                    id=2,
                    name="Test Name 2"
                )
            )
        error = cm.exception
        self.assertEqual(str(error),
            "The types of documents in a container should be the same" )

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_index_no_docs(self):

        docs = SolrDocs()

        with mock.patch('wukong.api.SolrAPI.update') as mock_update:
            mock_update.return_value = {}
            docs.index()

        assert not mock_update.called

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_index_with_docs(self):

        docs = SolrDocs()
        docs.add(
            FakeSolrDoc(
                id=1,
                name="Test Name 1"
            )
        )
        docs.add(
            FakeSolrDoc(
                id=2,
                partial_update=True,
                name="Test Name 2"
            )
        )

        with mock.patch('wukong.api.SolrAPI.update') as mock_update:
            mock_update.return_value = {}
            docs.index()

        mock_update.assert_called_once_with([
            {"id":1, "name":"Test Name 1"},
            {"id":2, "name":{"set":"Test Name 2"}}
        ])

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_index_commit(self):

        docs = SolrDocs()
        docs.add(
            FakeSolrDoc(
                id=1,
                name="Test Name 1"
            )
        )
        docs.add(
            FakeSolrDoc(
                id=2,
                partial_update=True,
                name="Test Name 2"
            )
        )
        with mock.patch('wukong.api.SolrAPI.update') as mock_update:
            with mock.patch('wukong.api.SolrAPI.commit') as mock_commit:
                mock_commit.return_value = {}
                docs.index(commit=True)

        assert mock_commit.called

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_delete_no_docs(self):

        docs = SolrDocs()

        with mock.patch('wukong.api.SolrAPI.delete') as mock_delete:
            mock_delete.return_value = {}
            docs.delete()

        assert not mock_delete.called

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdocs_delete_with_docs(self):

        docs = SolrDocs()
        docs.add(
            FakeSolrDoc(
                id=1,
                name="Test Name 1"
            )
        )
        docs.add(
            FakeSolrDoc(
                id=2,
                partial_update=True,
                name="Test Name 2"
            )
        )

        with mock.patch('wukong.api.SolrAPI.delete') as mock_delete:
            mock_delete.return_value = {}
            docs.delete()

        mock_delete.assert_called_once_with("id", "(1 2)", commit=False)

    @patch('wukong.api.SolrAPI.get_schema', mock_schema)
    def test_models_solrdoc_schema(self):

        doc = FakeSolrDoc(id=1)
        assert doc.schema == {
            'fields': [
                {'type': 'int', 'name': 'id'},
                {'type': 'string', 'name': 'name'},
                {'type': 'string', 'name': 'zip'},
                {'type': 'string', 'name': 'test_solrdoc_list', 'multiValued': True}],
                'uniqueKey': 'id',
                'dynamicFields': [
                    {'type': 'int', 'name': '*_d1'},
                    {'type': 'string', 'name': '*_d2'}]
        }
